@@ -1,5 +1,5 @@
 package WebService::GData::YouTube::YT::Incomplete;
-use WebService::GData::YouTube::Node;
+use WebService::GData::YouTube::YT;
 
 set_xml_meta(
   is_parent =>0
