@@ -1,5 +1,5 @@
 package WebService::GData::YouTube::YT::State;
-use WebService::GData::YouTube::Node;
+use WebService::GData::YouTube::YT;
 
 set_xml_meta(
   attributes=>[qw(name reasonCode helpUrl)]
